﻿using System;
using System.Drawing;
using System.IO;
using System.Xml.Linq;

namespace Core.Utilities 
{
    public static partial class ImageUtility
    {
        /// <summary>
        /// Ritorna una icona del Set Datagraph data una costante di colore e una percentuale (0-100) di trasparenza.
        /// </summary>
        /// <param name="imageKey">Nome dell'immagine <see cref="IconsKey"/></param>
        /// <param name="color">Costante del colore <see cref="Color"/></param>
        /// <param name="trasparency">Percentuale di trasparenza (default nessuna = 0)</param>
        /// <returns>Immagine vettoriale del Set Datagraph con colore e trasparenza specificati.</returns>
        public static string GetAwesome(string imageKey, Color color, int trasparency = 0) {
            return GetAwesome(imageKey, ColorUtility.ColorToHex(color), trasparency);
        }

        /// <summary>
        /// Ritorna una icona del Set Datagraph con una percentuale (0-100) di trasparenza.
        /// </summary>
        /// <param name="imageKey">Nome dell'immagine <see cref="IconsKey"/></param>
        /// <param name="trasparency">Percentuale di trasparenza (default nessuna = 0)</param>
        /// <returns>Immagine vettoriale del Set Datagraph con colore di default e trasparenza specificata.</returns>
        public static string GetAwesome(string imageKey, int trasparency = 0) {
            return GetAwesome(imageKey, null, trasparency);
        }

        /// <summary>
        /// Ritorna una icona del Set Datagraph data una costante di colore in esadecimale e una percentuale (0-100) di trasparenza.
        /// </summary>
        /// <param name="imageKey">Nome dell'immagine <see cref="IconsKey"/></param>
        /// <param name="colorHexOrThemeColor">Costante del colore nel formato esadecimale o nome del colore nel tema corrente prefissato da @ (es: @activeCaptionText)</param>
        /// <param name="trasparency">Percentuale di trasparenza (default nessuna = 0)</param>
        /// <returns>Immagine vettoriale del Set Datagraph con colore e trasparenza specificati.</returns>
        public static string GetAwesome(string imageKey, string colorHexOrThemeColor, int trasparency = 0) {
            return GetAwesome(imageKey, colorHexOrThemeColor, ColorUtility.AlphaToHex(trasparency));
        }

        /// <summary>
        /// Ritorna una icona del Set Datagraph data una costante di colore in esadecimale e una percentuale esadecimale (00-ff) di trasparenza.
        /// </summary>
        /// <param name="imageKey">Nome dell'immagine <see cref="IconsKey"/></param>
        /// <param name="colorHexOrThemeColor">Costante del colore nel formato esadecimale</param>
        /// <param name="trasparency">Percentuale di trasparenza nel formato esadecimale (default vuoto = nessuna trasparenza)</param>
        /// <returns>Immagine vettoriale del Set Datagraph con colore e trasparenza specificati.</returns>
        public static string GetAwesome(string imageKey, string colorHexOrThemeColor, string trasparency) {
            var res = $"resource.wx/Wisej.Ext.FontAwesome/{imageKey}.svg";
            if (colorHexOrThemeColor == null || colorHexOrThemeColor == string.Empty)
                return res;

            // Se hanno passato un nome di tema al posto di un colore (per passarlo usare come prefisso @)
            if (colorHexOrThemeColor.StartsWith("@"))
                return $"{res}?color={colorHexOrThemeColor.Substring(1)}";

            if (!colorHexOrThemeColor.StartsWith("#"))
                colorHexOrThemeColor = $"#{colorHexOrThemeColor}";

            return res + "?color=" + colorHexOrThemeColor;
        }

        /// <summary>
        /// Trasforma un array di byte in un oggetto Immagine.
        /// </summary>
        /// <param name="byteArrayIn">Array di byte da trasformare</param>
        /// <returns>Oggetto Image corrispondente all'array di Byte passato</returns>
        public static Image ByteArrayToImage(byte[] byteArrayIn)
        {
            if (byteArrayIn == null || byteArrayIn.Length == 0)
                return null;

            using (var memStream = new MemoryStream(byteArrayIn)) 
                return Image.FromStream(memStream);
        }

		/// <summary>
		/// Estrae come interi i valori di Larghezza e Altezza di una immagine .svg.
		/// </summary>
		/// <param name="svgContent">Contenuto dell'immagine .svg (letta come testo)</param>
		/// <returns>Ritorna gli interi che rappresentano Larghezza e Altezza dell'immagine .svg estratte dal file (-1 se il valore non è stato trovato)</returns>
		/// <example>
        ///     // Leggo le dimensioni dell'immagine
		///     (int width, int height) = ExtractSvgDimensions(svgContent);
        ///     ...
		/// </example>
		public static (int width, int height) ExtractSvgDimensions(string svgContent) {
            int width = -1;
            int height = -1;

			XDocument xDoc = XDocument.Parse(svgContent);
			XElement svgElement = xDoc.Root;

			string widthStr = svgElement.Attribute("width")?.Value;
			string heightStr = svgElement.Attribute("height")?.Value;

			// Rimuovi 'px' e converti in interi
			if (widthStr != null)
				width = int.Parse(widthStr.Replace("px", ""));
			
			// Rimuovi 'px' e converti in interi
			if (heightStr != null) 
			    height = int.Parse(heightStr.Replace("px", ""));
			
			return (width, height);			
		}
	}
}
